#include "playlistaspectlistviewmodel.h"
#include "debug.h"

enum Roles {
    PlayListIDRole = Qt::UserRole + 1,
    TrackNoRole = Qt::UserRole + 2,
    TitleRole = Qt::UserRole + 3,
    ArtistRole = Qt::UserRole + 4,
    AlbumRole = Qt::UserRole + 5,
    PlayTimeRole = Qt::UserRole + 6
};

PlayListAspectListViewModel::PlayListAspectListViewModel(PlayListAspectListViewRenderer *aspect)
    :	QAbstractListModel(aspect),
        aspect_(aspect)
{
    DGUARDMETHOD;
    QHash<int, QByteArray> names;
    names[PlayListIDRole] = "playListID";
    names[TrackNoRole] = "trackNo";
    names[TitleRole] = "title";
    names[ArtistRole] = "artist";
    names[AlbumRole] = "album";
    names[PlayTimeRole] = "length";
    setRoleNames(names);
}

int PlayListAspectListViewModel::rowCount(const QModelIndex &parent) const
{
    return aspect_->itemList()->count();
}

int PlayListAspectListViewModel::columnCount(const QModelIndex &parent) const
{
    return 6;
}

QVariant PlayListAspectListViewModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    PlayListItemList *itemList = aspect_->itemList();

    if (index.row() >= itemList->size())
        return QVariant();

    PlayListItem *item = itemList->at(index.row());
    Media *currentMedia = item->media();

    switch (role)
    {
    case Qt::DisplayRole:
    {
        QString t;
        switch (index.column())
        {
        case 0:
            t = QString::null;
            break;
        case 1:
            t = currentMedia->trackNo() + "  ";
            break;
        case 2:
            t = currentMedia->title();
            break;
        case 3:
            t = currentMedia->artist();
            break;
        case 4:
            t = currentMedia->album();
            break;
        case 5:
            t = currentMedia->playTime();
            break;
        }

        return t;
        break;
    }
    case PlayListIDRole:
        return (qulonglong)item->id();
    case TrackNoRole:
        return currentMedia->trackNo();
        break;
    case TitleRole:
        return currentMedia->title();
        break;
    case ArtistRole:
        return currentMedia->artist();
        break;
    case AlbumRole:
        return currentMedia->album();
        break;
    case PlayTimeRole:
        return currentMedia->playTime();
        break;
    default:
        return QVariant();
    }
}

QVariant PlayListAspectListViewModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (orientation == Qt::Horizontal)
    {
        QString t;
        switch (section)
        {
        case 0:
            t = QString::null;
            break;
        case 1:
            t = "No.";
            break;
        case 2:
            t = "Title";
            break;
        case 3:
            t = "Artist";
            break;
        case 4:
            t = "Album";
            break;
        case 5:
            t = "Length";
            break;
        }

        return t;
    }
    else
        return QVariant();
}

void PlayListAspectListViewModel::sort(int column, Qt::SortOrder order)
{
    DGUARDMETHOD;
    if (column != aspect_->sortedColumn())
        aspect_->setSortParams(column, order == Qt::AscendingOrder);
    else
        aspect_->setSortParams(aspect_->sortedColumn(), !aspect_->isSortAscending());

    DPRINTF("Header section %d clicked. Preparing playlist item index...", column);
}

void PlayListAspectListViewModel::rereadData()
{
    emit layoutChanged();
}
